/**************************************************************************
 * This file is part of Hypnotoad Live Wallaper.
 *
 *  Hypnotoad Live Wallaper is free software: you can redistribute it 
 *  and/or modify it under the terms of the GNU General Public License 
 *  as published by the Free Software Foundation, either version 3 of 
 *  the License, or (at your option) any later version.
 *
 *  Hypnotoad Live Wallaper is distributed in the hope that it will 
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  warranty of GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Hypnotoad Live Wallaper. If not, see <http://www.gnu.org/licenses/>.
 *************************************************************************/
package com.ancantus.HYPNOTOAD;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.nio.channels.FileChannel;
import java.util.UUID;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceManager;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.PreferenceActivity;
import android.provider.MediaStore;
import android.provider.MediaStore.MediaColumns;
import android.view.View;
import android.widget.Button;

public class Hypnotoad_LW_Preferences extends PreferenceActivity
{
	private static final String BACKGROUND_FILE = "background";
	private static final String TMP_FILE = "tmp";
	private static final int PICK_IMAGE = 1;
	//private static final int CROP_IMAGE = 2;	//put in later when time permits
	private Uri picasaUri = null;
	
	@SuppressWarnings("deprecation")	//addpreferencesfromresorces is depreciated, but needed for older versions
	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		
		//add the 2 xml files in (one build in preference and the 2nd to add buttons at the bottom)
		addPreferencesFromResource(R.xml.preferences);
		setContentView(R.layout.preference_layout);
		
		//setup the listener to start the background pick directly from the preference list
		Preference backgroundPreference = getPreferenceScreen().findPreference("background_color");
		backgroundPreference.setOnPreferenceChangeListener(backgroundChoiceListener);
		
		//setup listener to start the background choosing manually (also hide the button if there is no pic background)
		Button manualUpdate = (Button)findViewById(R.id.update_background);
		if (!(PreferenceManager.getDefaultSharedPreferences(this).getString("background_color", "0")).equals("2"))
		{
			manualUpdate.setVisibility(View.INVISIBLE);
		}
		manualUpdate.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) 
			{
				chooseImage();
			}
		});
		
		//setup listener for the gplDisplay button 
		Button gplDisplay = (Button)findViewById(R.id.dsp_gpl);
		gplDisplay.setOnClickListener(new View.OnClickListener() {
			//just open up a webpage to the gpl license. Displaying the txt is just too much work for the little use it will get
			@Override
			public void onClick(View v) 
			{
				Intent i = new Intent(Intent.ACTION_VIEW);
				i.setData(Uri.parse("http://www.gnu.org/licenses/gpl.txt"));	//hardcoding...BAD!
				startActivity(i);
			}
		});
	}
	
	OnPreferenceChangeListener backgroundChoiceListener = new OnPreferenceChangeListener() {
		@Override
		public boolean onPreferenceChange(Preference preference, Object newValue) 
		{
			if (newValue != null && newValue.toString().matches("2"))
			{	
				//set the manual background change button to visible
				findViewById(R.id.update_background).setVisibility(View.VISIBLE);
				
				//start the intents to pick an image
				chooseImage();
				return true;
			}
			else if (newValue != null && newValue.toString().matches("0") || newValue.toString().matches("1")) 
			{ 
				//set the manual background button invisible and return
				findViewById(R.id.update_background).setVisibility(View.INVISIBLE);
				return true; 
			}
			
			return false;
		}
	};
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) 
	{
		//make sure that its my request, and the data is there
		if(requestCode == PICK_IMAGE && data != null && data.getData() != null)
		{
			//get the image URI
			Uri imgUri = data.getData();
			
			if (imgUri != null)
			{
				//just not safe, need to copy from SDK and include it in (and I don't have the time to do that)
				//Intent cropIntent = new Intent("com.android.camera.action.CROP");
				//cropIntent.setType("image/*");
				//cropIntent.setData(imgUri);
				
				//now use the URI given, to get the path
				Cursor imgCursor = getContentResolver().query(imgUri, new String[] {MediaColumns.DATA, MediaColumns.DISPLAY_NAME}, null, null, null);
				
				//some android versions use uri of com.android instead of com.google.android
				if (imgUri.toString().startsWith("content://com.android.gallery3d.provider"))  
				{
					// use the com.google provider, not the com.android provider.
					imgUri = Uri.parse(imgUri.toString().replace("com.android.gallery3d","com.google.android.gallery3d"));
				}
				
				if (imgCursor != null)
				{
					imgCursor.moveToFirst();
					
					//if it is a pacasa image on Honeycomb and newer
					if (imgUri.toString().startsWith("content://com.google.android.gallery3d"))
					{
						if (imgCursor.getColumnIndex(MediaColumns.DATA) != -1)
						{
							picasaUri = imgUri;
							//download the image in a background thread (should have a progress bar or something)
							new Thread(new Runnable() {
								@Override
								public void run()
								{
									loadBackground(picasaUri);
								}
							}).start();
						}
					}
					else //its a regular file (local image)
					{
						copyBackground(imgCursor.getString(imgCursor.getColumnIndex(MediaColumns.DATA)));
						imgCursor.close();
					}
				}
				else if (imgUri != null && imgUri.toString().length() > 0)	//picasa img from older OS
				{
					picasaUri = imgUri;
					//download the image in a background thread (should have a progress bar or something)
					new Thread(new Runnable() {
						@Override
						public void run()
						{
							loadBackground(picasaUri);
						}
					}).start();
				}

			}
		}
		super.onActivityResult(requestCode, resultCode, data);	//send it up for further processing
	}
	
	//for code re-use, use intents to pick an image (lazyness == good programming techniques)
	private void chooseImage()
	{
		File tmpFile = getFileStreamPath(TMP_FILE);
		tmpFile.getParentFile().mkdirs();
		
		Intent picPickerIntent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);

		picPickerIntent.setType("image/*");
		startActivityForResult(picPickerIntent, PICK_IMAGE);
		
	}
	
	@SuppressLint("ParserError")
	private void copyBackground(String source)
	{
		//get shared prefs
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		
		//generate the random filename ending
		String oldFilePath = prefs.getString("pic_path", "");
		String newFileName = BACKGROUND_FILE + "-" + UUID.randomUUID();
		
		FileChannel src = null;
		FileChannel dst = null;
		try 
		{
			//get the input and output file channels
			src = new FileInputStream(new File(source)).getChannel(); //open source
			dst = openFileOutput(newFileName, Context.MODE_PRIVATE).getChannel(); //open background file in the internal storage directory
			
			//now copy src to the background file
			dst.transferFrom(src, 0, src.size());
			
			//finally close the 2 files
			src.close();
			dst.close();
		} 
		catch (Exception e) //have to close file sreams even if you throw an exception
		{
			if (src != null)
			{
				try 
				{
					src.close();
				}
				catch (IOException e1) {}
			}
			if (dst != null)
			{
				try 
				{
					dst.close();
				}
				catch (IOException e1) {}
			}
		}
			
		//now get the background file's path and put that in the pic_path field (maybe depreciate the pic_path setting altogether maybe?)
		SharedPreferences.Editor prefEditor = prefs.edit();
		prefEditor.putString("pic_path", getFilesDir().getAbsolutePath() + "/" + newFileName);
		prefEditor.commit(); //and save
		
		//if there is an old file to delete (and it isn't a file outside my internal storage dir) delete it
		if (oldFilePath != "" && oldFilePath.contains(getFilesDir().getAbsolutePath()))
		{
			deleteFile(oldFilePath.replace(getFilesDir().getAbsolutePath() + "/", ""));
		}
	}
	
	//run as a thread to pull picasa images and copy them to the device
	private void loadBackground(Uri bitmapUri)
	{
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		
		//generate the random filename ending
		String oldFilePath = prefs.getString("pic_path", "");
		String newFileName = BACKGROUND_FILE + "-" + UUID.randomUUID();
		
		InputStream src = null;
		OutputStream dst = null;
		try
		{
			dst = openFileOutput(newFileName, Context.MODE_PRIVATE);
			
			if (bitmapUri.toString().startsWith("content://com.google.android.gallery3d"))
			{
				src = getContentResolver().openInputStream(bitmapUri);
			}
			else
			{
				src = new URL(bitmapUri.toString()).openStream();
			}
			
			//copy all the bytes from src to dst
			byte[] buffer = new byte[1024];
			int len;
			while((len = src.read(buffer)) != -1)
			{
				dst.write(buffer, 0, len);
			}
			src.close();
			dst.close();
		}
		catch (Exception e) 
		{
			if (src != null)
			{
				try 
				{
					src.close();
				}
				catch (IOException e1) {}
			}
			if (dst != null)
			{
				try 
				{
					dst.close();
				}
				catch (IOException e1) {}
			}
		}
		//now get the background file's path and put that in the pic_path field (maybe depreciate the pic_path setting altogether maybe?)
		SharedPreferences.Editor prefEditor = prefs.edit();
		prefEditor.putString("pic_path", getFilesDir().getAbsolutePath() + "/" + newFileName);
		prefEditor.commit(); //and save
				
		//if there is an old file to delete (and it isn't a file outside my internal storage dir) delete it
		if (oldFilePath != "" && oldFilePath.contains(getFilesDir().getAbsolutePath()))
		{
			deleteFile(oldFilePath.replace(getFilesDir().getAbsolutePath() + "/", ""));
		}
	}
}
